#! /usr/bin/perl -w

# mailtools 0.2
#
# Tools for analysing mbox format mail spools.
#
# Copyright (C) 2000 Alan De Smet chaos@highprogrammer.com
# http://www.highprogrammer.com/alan/perl/#mailtools
#
# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program in file entitled "COPYING"; if
# not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA  02111-1307, USA.  A copy of the
# GNU General Public License is available at
# http://www.fsf.org/copyleft/gpl.html .
#

use strict;

use Mail::Message;

package Mail::DetectUserAgent;
#===============================================================================

sub simplify_version {
	local ($_) = @_;
	return $_ if not /^\d+\.\d+/;
	my($v) = /^(\d+\.\d+)/;
	return $v;
}

sub simplify_os {
	local ($_) = @_;
	my(%os) = (
		'Win32' => 'Windows',
		'WinNT' => 'Windows',
		'Win16' => 'Windows',
		'Win95' => 'Windows',
		'Windows NT 5.0' => 'Windows',
		'AIX 4.1.' => 'AIX',
		'AIX 370.' => 'AIX',
		'A/UX.' => 'A/UX',
		'BSDI V2-V4' => 'BSDI',
		'BSD 4.3' => 'BSD',
		'FreeBSD' => 'FreeBSD',
		'BSDI V1' => 'BSDI',
		'OpenBSD' => 'OpenBSD',
		'Convex.' => 'Convex',
		'DG/UX.' => 'DG/UX',
		'Bull DPX/2 B.O.S.' => 'Bull DPX/2 B.O.S.',
		'Sequent Dynix (not ptx).' => 'Sequent Dynix',
		'Sequent Dynix/PTX 1.4.' => 'Sequent Dynix',
		'Sequent Dynix/PTX.' => 'Sequent Dynix',
		'EP/IX SVR4' => 'EP/IX',
		'HP/UX 9.x (using gcc)' => 'HP/UX',
		'HP/UX 10 using gcc.' => 'HP/UX',
		'HP/UX.' => 'HP/UX',
		'HP/UX 10.' => 'HP/UX',
		'Solaris (using gcc)(guess)' => 'Solaris',
		'Solaris <= 2.4 (using gcc)' => 'Solaris',
		'Solaris >= 2.5 (using gcc)' => 'Solaris',
		'Solaris(guess)' => 'Solaris',
		'Solaris (version <= 2.4)' => 'Solaris',
		'Solaris >= 2.5' => 'Solaris',
		'SunOS 4.1.' => 'SunOS',
		'SunOS 4.1 (using gcc).' => 'SunOS',
		'SunOS 4.0.' => 'SunOS',
		'Ultrix 4 on DECStations (using gcc)' => 'Ultrix',
		'Interactive Systems Corp. Unix' => 'Interactive Systems Corp. Unix',
		'Linux.' => 'Linux',
		'Lynxos.' => 'Lynxos',
		'Machten' => 'Machten',
		'MiNT.' => 'MiNT',
		'PC-DOS' => 'PC-DOS',
		'NetBSD.' => 'NetBSD',
		'NeXT MACH 2.1' => 'NeXT MACH',
		'OS/2' => 'OS/2',
		'OSF/1' => 'OSF/1',
		'Mac OS X' => 'Mac OS X',
		'SCO 3.2v5.0.x' => 'SCO',
		'SCO 3.2vx.' => 'SCO',
		'SGI IRIX 4.0.1.' => 'SGI IRIX',
		'System V R4.' => 'System V',
		'Ultrix 4 on DECStations' => 'Ultrix',
		'Ultrix 4 on VAXStations' => 'Ultrix',
		'UnixWare 2.x' => 'UnixWare',
	);

	return $os{$_} if exists $os{$_};
	return $_;
}

sub pine_os {
	my($short) = @_;
	my(%os) = (
		'A32' => 'AIX 4.1.',
		'A41' => 'AIX 4.1.',
		'AIX' => 'AIX 370.',
		'AUX' => 'A/UX.',
		'BS2' => 'BSDI V2-V4',
		'BSD' => 'BSD 4.3',
		'BSF' => 'FreeBSD',
		'BSI' => 'BSDI V1',
		'BSO' => 'OpenBSD',
		'CVX' => 'Convex.',
		'D-G' => 'DG/UX.',
		'DPX' => 'Bull DPX/2 B.O.S.',
		'DYN' => 'Sequent Dynix (not ptx).',
		'EPX' => 'EP/IX SVR4',
		'GH9' => 'HP/UX 9.x (using gcc)',
		'GHP' => 'HP/UX 10 using gcc.',
		'GSO' => 'Solaris (using gcc)(guess)',
		'GS4' => 'Solaris <= 2.4 (using gcc)',
		'GS5' => 'Solaris >= 2.5 (using gcc)',
		'GSU' => 'SunOS 4.1 (using gcc).',
		'GUL' => 'Ultrix 4 on DECStations (using gcc)',
		'HPP' => 'HP/UX.',
		'HPX' => 'HP/UX 10.',
		'ISC' => 'Interactive Systems Corp. Unix',
		'LNX' => 'Linux.',
		'LYN' => 'Lynxos.',
		'MCT' => 'Machten',
		'MNT' => 'MiNT.',
		'MSC' => 'PC-DOS',
		'NEB' => 'NetBSD.',
		'NXT' => 'NeXT MACH 2.1',
		'OS2' => 'OS/2',
		'OSF' => 'OSF/1',
		'OSX' => 'Mac OS X',
		'PT1' => 'Sequent Dynix/PTX 1.4.',
		'PTX' => 'Sequent Dynix/PTX.',
		'S40' => 'SunOS 4.0.',
		'SC5' => 'SCO 3.2v5.0.x',
		'SCO' => 'SCO 3.2vx.',
		'SGI' => 'SGI IRIX 4.0.1.',
		'SOL' => 'Solaris(guess)',
		'SO4' => 'Solaris (version <= 2.4)',
		'SO5' => 'Solaris >= 2.5',
		'SUN' => 'SunOS 4.1.',
		'SV4' => 'System V R4.',
		'ULT' => 'Ultrix 4 on DECStations',
		'UW2' => 'UnixWare 2.x',
		'VUL' => 'Ultrix 4 on VAXStations',
	);

	return $os{$short} if exists $os{$short};
	return "Unknown ($short)";
}

sub sniff_user_agent_guess {
	my($mess) = @_;
	if(not defined $mess) { return (); }

	local $_;
	my($author, $name, $os, $version);

	$_ = $mess->header('X-MimeOLE');
	if(defined $_)
	{
		($version, $name, $os, $author) = /Microsoft MimeOLE V?([\d\.]+)/i;
		if(defined $version) {
			$author = 'Microsoft(guess)';
			return ($author, $name, $os, $version);
		}
	}

	$_ = $mess->header('Reply-To');
	if(defined $_)
	{
		if(/^majordomo?/i) {
			return('Great Circle Associates(guess)',
				'Majordomo(guess)', undef, undef);
		}
	}


	$_ = $mess->header('X-Mailman-Version');
	if(defined $_)
	{
		return('Mailman Cabal(guess)',
			'Mailman(guess)', undef, $_);
	}

	$_ = $mess->header('From');
	if(defined $_)
	{
		if(/^Mail Delivery Subsystem/) {
			return('Sendmail(guess)',
				'Sendmail(guess)', undef, undef);
		}
	}


	return ();
}

sub sniff_user_agent_from_message_id {
	local($_) = @_;

	if(not defined $_) {
		return ();
	}

	if(/^<Pine\./) {
		my($name) = 'Pine';
		my($author) = 'University of Washington';

		my($os, $ver);

		($os, $ver) = /^<Pine\.([^.]{3,3})\.([^.]+\.[^.]+)/;
		if(defined $ver) {
			return ($author, $name, pine_os($os), $ver);
		}

		($os, $ver) = /^<Pine\.([^.]+\.[^.]+)/;
		if(defined $ver) {
			return ($author, $name, pine_os($os), $ver);
		}
		
		return($author, $name, undef, undef);
	}

	if(/mail\.yahoo\.com/) {
		return ('Yahoo(guess)', 'YahooMail', undef, undef);
	}

	if(/hotmail\.com/) {
		return ('Microsoft(guess)', 'Hotmail', undef, undef);
	}

	return ();
}

sub sniff_user_agent {

	my($mess) = @_;
	if(not defined $mess) { return (); }

	my($useragent) = $mess->header('User-Agent');

	$useragent = $mess->header('X-Mailer') if not defined $useragent;

	
	if(not defined $useragent) {
		my(@results) =
			sniff_user_agent_from_message_id($mess->header('Message-ID'));
		if($#results == -1) {
			@results = sniff_user_agent_guess($mess);
		}
		return @results;
	}


	my($author, $name, $os, $version);
	local $_ = $useragent;

	# Pegasus Mail
	my($pegmail_author) = 'David Harris';
	($name, $os, $version) = /(Pegasus Mail) for (Win32) \(v(.+)\)/;
	return ($pegmail_author, $name, $os, $version) if defined $name;
	($name, $os, $version) = /(Pegasus\s*Mail)/i;
	return ($pegmail_author, $name, $os, $version) if defined $name;

	# Perl's Mail::Sender
	my($perlmail_author) = 'Jan Krynicky';
	($name, $version, $os) =
		/(Perl Mail::Sender)\s*(?:Version:?\s+)?([\d.]+)?/i;
	return ($perlmail_author, $name, $os, $version) if defined $name;

	# Mutt
	my($mutt_author) = 'Michael R. Elkins';
	($name, $version, $os) = /^(Mutt)[\/ ]+(.+)/i;
	return ($mutt_author, $name, $os, $version) if defined $name;
	($name, $os, $version) = /^(Mutt)/i;
	return ($mutt_author, $name, $os, $version) if defined $name;

	# Mozilla (Netscape)
	if(not /compatible/i) {
		my($mozilla_author) = 'Netscape';
		($name, $version, $os) = /^(Mozilla)\s+(\S+).+\(X11;\s+\w;\s*([^\d;\)]+)/i;
		return ($mozilla_author, $name, $os, $version) if defined $name;

		($name, $version, $os) = /^(Mozilla)\s+(\S+).+\(([^;\)]+)/i;
		return ($mozilla_author, $name, $os, $version) if defined $name;

		($name, $os, $version) = /^(Mozilla)/i;
		return ($mozilla_author, $name, $os, $version) if defined $name;
	}

	# Microsoft (multiple similar products)
	if(/^Microsoft/) {
		($author, $name, $os, $version) =
				/(Microsoft)[-\s]+(Outlook[-\s]+Express)[-\s]+
				(?:for[-\s]+)?(Macintosh)[-\s]+(?:Edition)?[\/-\s]+(.+)/ix;
		return ($author, $name, $os, $version) if defined $name;

		($author, $name, $os, $version) = /(Microsoft)-(Outlook-Express)-(Macintosh)-Edition\/([\d\.]+)/;
		return ($author, $name, $os, $version) if defined $name;

		($author, $name, $version, $os) = /(Microsoft)\s+(.+?)\s+([\d\.]+)/;
		return ($author, $name, $os, $version) if defined $name;

		return ($_, undef, undef);
	}
	my($ims_author) = 'Microsoft';
	($name, $version, $os) = /^(Internet Mail Service)\s+\((.+)\)/i;
	return ($ims_author, $name, $os, $version) if defined $name;
	($name, $version, $os) = /^(Internet Mail Service)/i;
	return ($ims_author, $name, $os, $version) if defined $name;

	# Mulberry
	my($mulberry_author) = 'Cyrusoft';
	($name, $version, $os) = /^(Mulberry)[\/ ]+(\S+)\s+\(([^\)]+?)(?:\s*Demo)?\)/i;
	return ($mulberry_author, $name, $os, $version) if defined $name;
	($name, $version, $os) = /^(Mulberry)[\/ ]+(\S+)/i;
	return ($mulberry_author, $name, $os, $version) if defined $name;
	($name, $os, $version) = /^(Mulberry)/i;
	return ($mulberry_author, $name, $os, $version) if defined $name;


	# Qualcomm
	my($eudora_author) = 'Qualcomm';
	if(/^QUALCOMM/i) {
		($os, $name, $version) = /^QUALCOMM\s*(\S+)\s+(.+)\s+version\s+(.+)/i;
		return ($eudora_author, $name, $os, $version) if defined $name;

		($os, $name, $version) = /^QUALCOMM\s*(\S+)\s+(.+)\s+([\d\.]+)/i;
		return ($eudora_author, $name, $os, $version) if defined $name;

		($os, $name, $version) = /^QUALCOMM\s*(.+)/i;
		return ($eudora_author, $name, $os, $version);
	}
	($os, $name, $version, $author) = 
			/^(\S+)\s+(Eudora(?:\s*(?:Pro|Light))?)
				\s+version\s+(.+)/ix;
	return ($eudora_author, $name, $os, $version) if defined $name;


	# exmh
	my($exmh_author) = 'exmh(unknown)';
	($name, $version, $os) = /^(exmh)(?:\s+version)?\s+(.+)/i;
	return ($exmh_author, $name, $os, $version) if defined $name;
	($name, $os, $version) = /^(exmh)/i;
	return ($exmh_author, $name, $os, $version) if defined $name;


	# Claris
	($author, $name, $version, $os) = /^(Claris)\s*(Emailer)\s+(.+)/i;
	return ($author, $name, $os, $version) if defined $name;
	($author, $name, $os, $version) = /^(Claris)\s*(Emailer.+)/i;
	return ($author, $name, $os, $version) if defined $name;


	# AOL
	my($aol_author) = 'AOL';
	($os, $name, $version) = /^(\S+)\s+(AOL)(?:\s+sub)?\s+(.+)/i;
	return ($aol_author, $name, $os, $version) if defined $name;
	($name, $os, $version) = /^(\S*\s*AOL)\b/i;
	return ($aol_author, $name, $os, $version) if defined $name;

	# MondoMail
	my($mondo_author) = 'Acme Technologies';
	($name, $version, $os) = /^(MondoMail(?:\s+Pro)?)\s+(.+)/i;
	return ($mondo_author, $name, $os, $version) if defined $name;
	($name, $version, $os) = /^(MondoMail)\b/i;
	return ($mondo_author, $name, $os, $version) if defined $name;


	# fastmail (??)
	($name, $version, $os, $author) = /^(fastmail) \[version ([^\]+])/i;
	return ($author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(fastmail)\b/i;
	return ($author, $name, $os, $version) if defined $name;

	# ccMail (??)
	my($ccmail_author) = 'Lotus(guess)';
	($name, $version, $os, $author) = /^(ccMail)\b Link to SMTP (.+)/i;
	return ($ccmail_author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(ccMail)\b/i;
	return ($ccmail_author, $name, $os, $version) if defined $name;

	# MediaMail, SGI's "subset of Z-Mail" (I don't understand it...)
	my($zmail_author) = 'SGI(guess)';
	($version, $name, $os, $author) = /^Z-Mail-SGI \((\S+).+(MediaMail)/i;
	return ($zmail_author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^Z-Mail.+(MediaMail)/i;
	return ($zmail_author, $name, $os, $version) if defined $name;

	# XFMail
	my($xfmail_author) = 'Gennady B. Sorokopud(guess)';
	($name, $version, $os, $author) = /^(XFMail)\s+(\S+).+\s+on\s+(.+)/i;
	return ($xfmail_author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(XFMail)\s+(\S+)/i;
	return ($xfmail_author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(XFMail)\b/i;
	return ($xfmail_author, $name, $os, $version) if defined $name;

	# MessageMedia - UnityMail
	my($unitymail_author) = 'MessageMedia';
	($name, $version, $os, $author) = /^(UnityMail)\b/i;
	return ($unitymail_author, $name, $os, $version) if defined $name;

	# USANET web-mailer ??
	($name, $version, $os, $author) = /^(USANET web-mailer)\s+\((.+)\)/i;
	return ($author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(USANET web-mailer)/i;
	return ($author, $name, $os, $version) if defined $name;

	# TkReq 
	($name, $version, $os, $author) = /^(TkReq)\s+v?([\d\.]+)/i;
	return ($author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(TkReq)/i;
	return ($author, $name, $os, $version) if defined $name;

	# SMTPit - FilerMaker Pro EMail Plugin 
	($name, $version, $os, $author) =
		/^(SMTPit - FilerMaker Pro EMail Plugin) \(ver\. (.+)\)/i;
	return ($author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) =/^(SMTPit - FilerMaker Pro EMail Plugin)/i;
	return ($author, $name, $os, $version) if defined $name;

	# Elm  ELM [version 2.4ME++ PL40 (25)]
	my($elm_author) = 'Elm USENET Community Trust';
	($name, $version, $os, $author) = /^(ELM)\s+\[(?:version\s+)?(.+)\]/;
	return ($elm_author, $name, $os, $version) if defined $name;
	($name, $version, $os, $author) = /^(ELM)\b/i;
	return ($elm_author, $name, $os, $version) if defined $name;

	return (undef, $_, undef, undef);
}


sub new {
	my $class = shift;
	$class = ref($class) if ref($class);
	my $mess = shift;

	my($author, $name, $os, $version) = sniff_user_agent($mess);

	bless my $self = {
		'author' => $author,
		'name' => $name,
		'os' => $os,
		'version' => $version,
	}, $class;
	return $self;
}

sub author {
	my($self) = shift;
	return $self->{'author'};
}

sub useragent {
	my($self) = shift;
	return $self->{'name'};
}

sub os {
	my($self) = shift;
	return $self->{'os'};
}

sub simple_os {
	my($self) = shift;
	my($os) = $self->os();
	return undef if not defined $os;
	return simplify_os($os);
}

sub version {
	my($self) = shift;
	return $self->{'version'};
}

sub simple_version {
	my($self) = shift;
	my($v) = $self->version();
	return undef if not defined $v;
	return simplify_version($v);
}

1;
