#! /usr/bin/perl -w

# mailtools 0.2
#
# Tools for analysing mbox format mail spools.
#
# Copyright (C) 2000 Alan De Smet chaos@highprogrammer.com
# http://www.highprogrammer.com/alan/perl/#mailtools
#
# This software is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public
# License along with this program in file entitled "COPYING"; if
# not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA  02111-1307, USA.  A copy of the
# GNU General Public License is available at
# http://www.fsf.org/copyleft/gpl.html .
#

use strict;

#===============================================================================

package Mail::Message;

sub new {
	my $class = shift;
	$class = ref($class) if ref($class);
	my @message = @_;

	# TODO: empty message?  One line (whole body)?

	my($errors) = 0;
	my(%headers, $body, $line);

	my($from_) = $message[0];
	if(not defined $from_ or not $from_ =~ /^From /) {
		$errors++;
	} else {
		shift @message;
	}

	# Parse headers
	my($previous_value) = undef;
	while(defined ($line = shift @message) )
	{
		if($line =~ /^\S+:/) { # Header start
			my($key, $value) = ($line =~ /^(\S+):\s*(.*?)\s*$/);
			$key = canonize_key($key);
			if(not exists $headers{$key}) {
				$headers{$key} = ( );
			}
			push @{ $headers{$key} }, $value;
			$previous_value = \ $headers{$key}[$#{$headers{$key}}];
		} elsif($line =~ /^\s+\S/) { # Continuation
			if(not defined $previous_value) {
				$errors++;
				next;
			}
			my($value) = ($line =~ /^\s+(.+?)\s*$/);
			${$previous_value} .= " $value";
		} elsif($line =~ /^[\r\n]*$/) {
			last;
		} else {
			$errors++;
			print STDERR "Unable to process line: $line";
		}
	}

	if($#message >= 0) {
		$body = join('', @message);
	}

	bless my $self = {
		'errors' => $errors,
		'headers' => \%headers,
		'bodyplain' => $body,
	}, $class;

	return $self;
}

sub canonize_key {
	my($key) = shift;
	if(not defined $key) { return undef; }

	$key = "\L$key";

	my(%official) = (
		# Special cases for keys that look funny if using brute force method
		'message-id' => 'Message-ID',
	);

	if(exists $official{$key}) {
		return $official{$key};
	}

	my(@chunks) = split(/-/, $key);
	foreach my $chunk (@chunks) {
		$chunk = "\u$chunk";
	}
	return join('-', @chunks);
}

sub header {
	my($self) = shift;
	my($header) = shift;
	$header = canonize_key($header);
	my($headers) = $self->{'headers'};
	if(not defined $headers or not exists $headers->{$header}) {
		return wantarray ? () : undef;
	}

	if(wantarray) {
		return @{$headers->{$header}};
	}

	return $headers->{$header}[0];
}

sub subject {
	return header($_[0], 'Subject');
}

sub message_id {
	return header($_[0], 'Message-ID');
}

sub strong_references {
	my($self) = shift;
	my($references) = $self->header('References');
	if(not defined $references) {
		my($references) = $self->header('In-Reply-To');
	}
	if(not defined $references) {
		return ();
	}

	my(@parents) = ($references =~ /(<[^>]+>)/g);

	# TODO: Strip out garbage, reduce to list of <.+@.+>, remove things
	# that look like
}

1;
